# Contributing
## A note on documentation

Since this is meant to be the kind of code that you can just "dive into" usefulpy has to be well documented.

### Triple quotes

Triple quotes are only to be used when writing docstrings or long strings, use `#` for multi line commenting

### Comments

Single `#` for comments, These include comments on how something works, what something is, or a mathematical proof!

from 'quaternion.py'

```python
# My thought process is such:
# e**n = 1+x+x^2/2!+x^3/3!...
# so o**s = e**(ln(o)*s)
num = ln(other)*self
```

from 'validation.py'

```python
#for easier reference of the function type.
function = type(lambda: None)
```

Double `##` for markers that mark something that needs to be done. Additional stuff like `BUG`, `TODO`, or `FIXME` may be there as well.
This is also used for deactivating statements that were only there for the purposes of debuging

from (an older version of) 'quaternion.py'

```python
##TODO: I need to get log/ln for these first...
raise NotImplementedError('Raising quaternions to non-real powers has not been implemented yet')
```

from 'quaternion.py'

```python
##print(a, b, c, d)
```

Triple `###` on each side of the name of a segment of code; these serve to partition the code into several parts. Search for the `###` to find the headers of these sections.

from '\_\_init\_\_.py' for usefulpy

```python
### IMPORTS ###
from . import validation, formatting, mathematics, quickthreads, decorators
from .IDE import ide, run_path, startup
```

## Versionieering

We use [semantic versioning](https://semver.org/)

Versions are formatted as x.y.z, or MAJOR.MINOR.PATCH.

In summary:

An update to z (a patch) means refers to small changes, fixed bugs, improved performance, _et cetera_.

An update to y (a minor update) refers to larger changes that include adding functions or changing their jobs. This type of change is not backward compatible so something that is supposed to be possible in the new version was not possible in the version before. But something that used to work still works

An update to x (a major update) refers to an update that is not forward compatible so something that used to work no longer works.

## What to do

You've opened a branch to contribute, but don't know what to do? I've got a lot to do... here is some of it.

### TODO.md

An auto generated file appears in usefulpy named `TODO.md`, this is generated by a code that I wrote that finds instances of the `##` in the entire usefulpy code.
There you will find _a lot_ to do.

### Plans

There already some stuff planned out, some visible from the placeholders. More 3d projection stuff would be nice. For the sake of efficiency I would like would be to move some stuff over to some form of compiled code. Sifting through and improving the code could make enormous improvements in quality.

A particualar thing that must be done is the finishing and merging of the `eq`, `mathfunc`, and `Expression` classes. These were started at different points for different reasons, but much of their functionality is very similar, and none of them is really finished. The plan is to merge this into a single `mathfunc` class.

## Bugs

Bugs are reported [here](https://github.com/Augustin007/usefulpy/issues).